<?php

/**
 * @class Session
 * @brief Represent a XeeK victim session
 */
class Session extends DBObject {

  //ID of the chain
  protected $audit_id;
  
  //ID of the current instruction
  protected $instruction_id;
  
  //Cookie
  protected $cookie;
  
  //Public IP of the victim
  protected $ip;
  
  //User agent
  protected $useragent;
  
  //Created at
  protected $created_at;
  
  //Updated at
  protected $last_seen;
  
  
  //Audit object
  protected $audit;
  
  //Timeout in seconds before considering a session as offline
  const ONLINE_TIMEOUT = 10;
  
  
  /**
   * @brief Constructor.
   * @param $args Associative array: property => value
   * @param $isFetch True if the object has been fetched from the database
   */
  public function __construct($args, $isFetch = false) {
  
    //Call the parent constructor
    parent::__construct($args, $isFetch);
    
    $this->audit = null;
    
    //Enable serialization
    $this->setSerializable(array('id', 'audit_id', 'instruction_id', 'xeek_cookie', 'ip', 'useragent', 'created_at', 'last_seen'));
  }
  
  /**
   * @brief Save the session
   * @param $prop_names properties we want to save, or null for all properties.
   */
  public function save($prop_names = null) {
    //Set the last_seen property
    $this->updateLastSeen();
    
    parent::save($prop_names);
  }
  
  
  /**
   * @brief Set the reference to the audit. IF no audit provided, try to fetch it using $this->audit_id
   * @param $audit Audit object
   */
  public function setAudit($audit = null) {
    if($audit == null){
      $this->audit = Audit::fetchById($this->audit_id);
    } else {
      $this->audit = $audit;
    }
  }
  
  
  
  /**
   * @brief Get next instructions to execute for this session
   * @return array of next instructions, or empry array if there is no one
   */
  public function getNextInstructions() {
    //Check audit
    if($this->audit == null) {
      throw new InvalidAudit();
    }
    
    //Fetch next instructions
    $instructions = $this->audit->queue->popNextInstructions($this->id, $this->instruction_id);
    
    //If there is one, update the current instruction id
    if(count($instructions) != 0) {
      $this->instruction_id = $instructions[count($instructions)-1]->id;
      $this->save();
    }
    
    return $instructions;
  }
  
  
  /**
   * @brief Update the last_seen property
   */
  public function updateLastSeen(){
    $this->last_seen = date("Y-m-d H:i:s");
  }
  
  
  /**
   * @brief Reset the session by setting the current Instruction ID to a specific value
   * @param $instruction_id Instruction ID to set
   */
  public function reset($instruction_id){
    $this->instruction_id = intval($instruction_id);
    $this->save();
  }
  
  
  /**
   * @brief Find a session given a cookie, IP address and user agent.
   * @param $xeek_cookie XeeK cookie
   * @param $ip public IP address of the victim
   * @param $useragent browser identifier
   * @result session object or null if no one corresponds
   */
  public static function find($xeek_cookie, $ip, $useragent) {
    return parent::fetchOne("xeek_cookie = '".MySQL::escape_string($xeek_cookie)."' ".
                            "AND ip = '".MySQL::escape_string($ip)."' ".
                            "AND useragent = '".MySQL::escape_string($useragent)."'");
  }
  
  /**
   * @brief List all sessions
   * @return array of sessions
   */
  public static function listAll(){
    return parent::fetchList(1, "created_at");
  }
  
  /**
   * @brief List sessions that are online
   * @param $audit_id audit id, 0 to get all sessions
   * @param $online true to get only online sessions
   * @return array of sessions
   */
  public static function listSessions($audit_id = 0, $online = false){
    $where = "1";
    if($audit_id != 0){
      $where .= " AND audit_id = '".intval($audit_id)."'";
    }
    if($online){
      $where .= " AND last_seen > SUBTIME(NOW(), ".intval(self::ONLINE_TIMEOUT).")";
    }
    return parent::fetchList($where, "created_at");
  }
  
  /**
   * @brief Delete all sessions
   * @param $audit_id audit id to clean. If not provided, delete all sessions.
   * @param return true
   */
  public static function cleanAll($audit_id = 0){
    $where = ($audit_id == 0) ? "1" : "audit_id = '".intval($audit_id)."'";
    static::deleteWhere($where);
    return true;
  }
  
  public static function getClass(){
    return "Session";
  }
  
  public static function getTable(){
    return "session";
  }
  
  public static function getAutoProps(){
    return array("audit_id", "instruction_id", "xeek_cookie", "ip", "useragent", "created_at", "last_seen");
  }

}

?>