<?php

/**
 * @class SessionManager
 * @brief Session management class
 */
class SessionManager {

  //Name of the cookie and parameter
  const SESSID_PARAM_NAME = "xeek_sessid";
  
  
  /**
   * @brief Create a session
   * @param $audit_id Audit id
   * @param $ip IP address
   * @param $useragent User agent
   * @return Session object
   */
  protected static function createSession($audit_id, $ip, $useragent) {
    //Generate a cookie
    $xeek_cookie = mt_rand();
    
    //Set the cookie
    setcookie(self::SESSID_PARAM_NAME, $xeek_cookie);
    
    //Create the session and save it
    $session = new Session(array( "audit_id" => $audit_id, 
                                  "xeek_cookie" => $xeek_cookie, 
                                  "ip" => $ip, 
                                  "useragent" => $useragent));
   
    $session->save();
    
    return $session;
  }
  
  
  /**
   * @brief Initialize a session: try to detect one, or create a new one.
   * @param $audit_token Audit token
   * @param $server_array $_SERVER array
   * @param $cookie_array $_COOKIE array
   * @param $get_array $_GET array
   * @return Session object, or null if the provided audit token is incorrect
   */
  public static function initSession($audit_token, $server_array, $cookie_array, $get_array) {
    
    //Check the audit token
    $audit = Audit::fetchByToken($audit_token);
    if($audit == null) {
      return null;
    }
    $audit_id = $audit->id;
    
    //Get User-agent and IP
    $useragent = $server_array['HTTP_USER_AGENT'];
    $ip = $server_array['REMOTE_ADDR'];
    
    //Try to detect a session
    $session = self::getSession($server_array, $cookie_array, $get_array);
    
    //If no session was detected, create one
    if($session == null){
      $session = self::createSession($audit_id, $ip, $useragent);
    }
    
    //Set the reference to the audit and save the session object
    if($session != null) {
      $session->setAudit($audit);
      $session->save();
    }
    
    return $session;
  }
  
  
  
  
  /**
   * @brief Detect a session
   * @param $server_array $_SERVER array
   * @param $cookie_array $_COOKIE array
   * @param $get_array $_GET array
   * @return Session object, or null if no session can be detected
   */
  public static function getSession($server_array, $cookie_array, $get_array) {
    
    //Get User-agent and IP
    $useragent = $server_array['HTTP_USER_AGENT'];
    $ip = $server_array['REMOTE_ADDR'];
    
    //Try to detect a session using cookies
    $session = null;
    if(array_key_exists(self::SESSID_PARAM_NAME, $cookie_array)){
    
      //If there is a cookie, read
      $xeek_cookie = $cookie_array[self::SESSID_PARAM_NAME];
      
      //Try to find a session
      $session = Session::find($xeek_cookie, $ip, $useragent);
    }
    
    //If it didn't work, use the GET parameter
    if(!$session && array_key_exists(self::SESSID_PARAM_NAME, $get_array)){
    
      //If there is a param, read it
      $param = $get_array[self::SESSID_PARAM_NAME];
      
      //Try to find a session
      $session = Session::find($param, $ip, $useragent);
    }
    
    //If the session was detected, set the reference to the audit and save the session object
    if($session != null) {
      $session->setAudit();
      $session->save();
    
    } else {
      
      //setcookie(self::SESSID_PARAM_NAME, "", time() - 3600); //TODO test
    }
    
    return $session;
  }

}

?>