<?php

require_once(dirname(__FILE__)."/MySQL.class.php");

/**
 * @class DBObject
 * @brief Represent an object stored in a database
 */
abstract class DBObject {

  //Token length
  const TOKEN_LENGTH = 8;

  //Id of the object
  public $id = 0;
  
  //Serializable properties
  protected $seralizable_props = null;
  
  
  /**
   * @brief Constructor
   * @param $params Array of parameters to initialize
   * @param $isFetch True if the object has been fetched from the database
   */
  public function __construct($params = null, $isFetch = false) {
    if($params == null) {
      $params = array();
    }
    
    foreach($params as $p => $v){
      $this->$p = $v;
    }
    
    if($isFetch){
      $this->postFetch();
    }
  }
  
  
  /**
   * @brief Main function for saving the object properties into the database
   * @param $prop_names properties we want to save, or null for all properties.
   */
  public function save($prop_names = null) {
    
    if($prop_names == null){
      $prop_names = static::getAutoProps();
    }
    
    if(!is_array($prop_names)){
      $prop_names = array($prop_names);
    }
    
    //Retreieve the value for each property
    $props = array();
    $all_props = get_object_vars($this);
    foreach($prop_names as $p){
      $props[$p] = $all_props[$p];
    }
    
    //Handle created_at
    if(array_key_exists("created_at", $props)) unset($props["created_at"]);
    
    
    //Perform the save
    return $this->_save($props);
  }
  
  
  /** 
   * @brief Actually save the object (update or insert)
   * @param $props Object properties that will be saved into the database
   */
  protected function _save($props) {
    
    //Insert or update
    if($this->id == 0) {
      $this->id = MySQL::insert(static::getTable(), $props);
    } else {
      MySQL::update(static::getTable(), $this->id, $props);
    }
    
    return $this->id;
  }

  
  /**
   * @brief Execute something after the object has been feteched from the database (decode fields, etc)
   */
  protected function postFetch() { }
  
  
  /**
   * @brief Get the list of all audits
   * @param $where_clause WHERE clause
   * @param $order_by ORDER BY clause
   * @param $limit LIMIT clause
   * @return array of objects
   */
  public static function fetchList($where_clause = 1, $order_by = "id", $limit = "") {
    $tab = MySQL::select(static::getTable(), static::getPropNames(), $where_clause, $order_by, $limit);
    
    $res = array();
    $i = 0;
    
    $class = static::getClass();
    
    //For each result, create the object and fill in its properties
    foreach($tab as $t){
      $res[$i++] = new $class($t, true);
    }
    
    return $res;
  }
  
  
  /**
   * @brief Fetch one object
   * @param $where_clause WHERE clause
   * @param $order_by ORDER BY clause
   * @return object or null if no one corresponds
   */
  public static function fetchOne($where_clause = 1, $order_by = "id") {
    $tab = MySQL::selectOne(static::getTable(), static::getPropNames(), $where_clause, $order_by);
    
    if($tab == null) return null;
    
    $res = array();
    $i = 0;
    
    //Create the object and fill in its properties
    $class = static::getClass();    
    return new $class($tab, true);
  }
  
  
  /**
   * @brief Find an object given its id
   * @param id
   * @return object, or null if no one corresponds
   */
  public static function fetchById($id) {
    return static::fetchOne("id = '".intval($id)."'");
  }
  
  
  /**
   * @brief Find an object using a join query
   * @param $tables Tables to append in the FORM clause of the query. It can be an array of table names, or a string if there is only one table.
   * @param $where_clause WHERE clause
   * @param $order_by ORDER BY clause
   * @param $limit LIMIT clause
   */
  public static function fetchWithJoin($tables, $where_clause = 1, $order_by = "", $limit = ""){
    
    if(!is_array($tables)){
      $tables = array($tables);
    }
    
    $tables = array_merge(array(static::getTable()), $tables);
    
    if(!$order_by){
      $order_by = static::getTable().".id";
    }
    
    $tab = MySQL::select($tables, static::getPropNames(), $where_clause, $order_by, $limit, true);
    
    $res = array();
    $i = 0;
    
    $class = static::getClass();
    
    //For each result, create the object and fill in its properties
    foreach($tab as $t){
      $res[$i++] = new $class($t, true);
    }
    
    return $res;
  
  }
  
  
  /**
   * @brief Find an object given its token (only for token-based objects)
   * @param token
   * @return object, or null if no one corresponds
   */
  public static function fetchByToken($token) {
    $token = MySQL::escape_string($token);
    return static::fetchOne("token = '".$token."'");
  }
  
  
  /**
   * @brief Generate a token (only for token-based objects)
   * @return token
   */
  protected static function genToken(){
    $length = self::TOKEN_LENGTH;
    $characters = "0123456789abcdefghijklmnopqrstuvwxyz";
    $string = "";    

    for ($p = 0; $p < $length; $p++) {
      $string .= $characters[mt_rand(0, strlen($characters) - 1)];
    }

    return $string;
  }
  
  
  /**
   * @brief Delete the object
   */
  public function delete() {
    MySQL::delete(static::getTable(), "id = '".intval($this->id)."'");
    $this->id = 0;
  }
  
  
  /**
   * @brief Delete objects based on a condition
   * @param where_clause condition
   */
  public static function deleteWhere($where_clause) {
    MySQL::delete(static::getTable(), $where_clause);
  }
  
  
  /**
   * @brief Get propertiy names for this object
   * @return array of property names
   */
  protected static function getPropNames() {
    $tab = array_merge(array("id"), static::getAutoProps());
    return $tab;
  }
  
  /**
   * @brief Enable serialization of the object
   * @param $props array of property names that are serializable
   */
  protected function setSerializable($props){
    $this->seralizable_props = $props;
  }
  
  
  /**
   * @brief Create an external representation of the object, to be handled by the API
   * @return array representing the object, or null if the object is not serializable
   */
  public function serialize(){
    if($this->seralizable_props != null){
      $a = array();
      foreach($this->seralizable_props as $p){
        $a[$p] = $this->$p;
      }
      return $a;
    } else {
      return null;
    }
  }
  
  
  
  /**
   * @brief Get the class name
   * @return class name
   */
  public static abstract function getClass();
  
  /**
   * @brief Get the table name
   * @return table name
   */
  public static abstract function getTable();
  
  /**
   * @brief Get the properties that have to be handled by the ORM
   * @return array of property names
   */
  public static abstract function getAutoProps();

}

?>
