#!/usr/bin/python
# -*- coding: iso-8859-15 -*-

import urllib

try:
  import simplejson as json
except:
  import json
from exceptions import *
from objects import *


class ClientAPI:

  def __init__(self, url, proxies = {}):
    """Constructor"""
    self._base_url = url
    self._proxies = proxies
    self._jsonencoder = json.JSONEncoder()
    self._jsondecoder = json.JSONDecoder()
    self._last_error = None

    
  
  def _request(self, http_method, method, params = None):
    """
    Send a request to the server, parse the JSON result and return it as a Python array
    @param http_method HTTP method ("GET" or "POST")
    @param method method to call
    @param params parameter dict
    @return Python object if everthing goes fine, None otherwise
    """
    if params:
      params = urllib.urlencode(params)
    
    if(http_method == "GET"):
      path = "/" + method
      if params:
        path += "?" + params
        
      try:
        conn = urllib.urlopen(self._base_url + path, proxies = self._proxies)
      except:
        raise LostConnection()
        
    elif(http_method == "POST"):
      path = "/" + method
      try:
        conn = urllib.urlopen(self._base_url + path, params, proxies = self._proxies)
      except:
        raise LostConnection()
    
    response = ""
    try:
      response = conn.read().encode('iso-8859-15')
      return self._jsondecoder.decode(response)
    except:
      raise ParsingError(response)
    
  
  ##############################################################
  # API Methods
  ##############################################################
  
  
  def getAudit(self, id):
    """
    Get an audit
    @param id audit id
    @return audit
    """
    res = self._request("GET", "audit/get", {'id': id})
    XeekAPIException.checkOrRaise(res)
    return APIAudit(res)
  

  def listAudits(self):
    """
    List audits
    @return audit list
    """
    res = self._request("GET", "audit/list")
    XeekAPIException.checkOrRaise(res)
    return APIObjectList(APIAudit, res)
    
    
  def deleteAudit(self, id):
    """
    Delete an audit
    @param id audit id
    @return true or false
    """
    res = self._request("POST", "audit/delete", {'id': id})
    XeekAPIException.checkOrRaise(res)
    return res
    
    
  def createAudit(self, name):
    """
    Create an audit
    @param name audit name
    @return audit
    """
    res = self._request("POST", "audit/add", {'name': name})
    XeekAPIException.checkOrRaise(res)
    return APIAudit(res)
    

  def getQueue(self, audit_id):
    """
    Get the queue of an audit
    @return audit list
    """
    res = self._request("GET", "queue/get", {'audit_id': audit_id})
    XeekAPIException.checkOrRaise(res)
    return APIObjectList(APIInstruction, res)
  
    
  def pushInstructionToQueue(self, audit_id, exploit_id, params, session_id = 0, prio = 0):
    """
    Push an instruction to the queue
    @param audit_id
    @param exploit_id
    @param params
    @param prio
    @result instruction
    """
    params = self._jsonencoder.encode(params)
    res = self._request("POST", 
                        "queue/instruction/add", 
                        {
                          "audit_id": audit_id, 
                          "exploit_id":exploit_id, 
                          "params":params, 
                          "session_id": session_id, 
                          "prio":prio
                        })
    XeekAPIException.checkOrRaise(res)
    return APIInstruction(res)
  
  
  def pushChainToQueue(self, audit_id, chain_id, params, prio = 0):
    """
    Push a chain to the queue
    @param audit_id
    @param chain_id
    @param params
    @param prio
    @result instruction
    """
    params = self._jsonencoder.encode(params)
    res = self._request("POST", "queue/chain/add", {"audit_id": audit_id, "chain_id":chain_id, "params":params, "prio":prio})
    XeekAPIException.checkOrRaise(res)
    return res
   
    
  def deleteInstruction(self, id):
    """
    Delete an instruction
    @param id instruction id
    @return true or false
    """
    res = self._request("POST", "queue/instruction/delete", {'id': id})
    XeekAPIException.checkOrRaise(res)
    return res
    
  
  def cleanQueue(self, audit_id):
    """
    Delete all instructions for a perticular audit
    @param audit_id audit id
    """
    res = self._request("POST", "queue/clean", {'audit_id': audit_id})
    XeekAPIException.checkOrRaise(res)
    return res
    
   
  def listSessions(self, audit_id=0, online=False):
    """
    List all sessions for all audits or for a perticular one
    @param audit_id audit id, 0 for all audits
    @param online True to get only online sessions, false otherwise
    @return array of sessions
    """
    res = self._request("GET", "session/list", {'audit_id': audit_id, 'online': int(online)})
    XeekAPIException.checkOrRaise(res)
    return APIObjectList(APISession, res)
  
  
  def cleanSessions(self, audit_id=0):
    """
    Delete all sessions for all audits or for a perticular one
    @param audit_id audit id, 0 for all audits
    """
    res = self._request("POST", "session/clean", {'audit_id': audit_id})
    XeekAPIException.checkOrRaise(res)
    return res
  
  
  def getSession(self, session_id):
    """
    Get a session
    @param session_id session id
    @return session
    """
    res = self._request("GET", "session/get", {'id': session_id})
    XeekAPIException.checkOrRaise(res)
    return APISession(res)
    
  
  def resetSession(self, session_id, instruction_id=0):
    """
    Reset a session
    @param session_id Session ID
    @param instruction_id Instruction ID, 0 by default
    @return session
    """
    res = self._request("POST", "session/reset", {'id': session_id, "instruction_id": instruction_id})
    XeekAPIException.checkOrRaise(res)
    return res
    
    
  def deleteSession(self, id):
    """
    Delete a session
    @param id session id
    @return true or false
    """
    res = self._request("POST", "session/delete", {'id': id})
    XeekAPIException.checkOrRaise(res)
    return res
    
  
  def getChain(self, chain_id):
    """
    Get a chain
    @param chain_id chain id
    @return chain
    """
    res = self._request("GET", "chain/get", {'id': chain_id})
    XeekAPIException.checkOrRaise(res)
    return APIChain(res)
    
  
  def listChains(self):
    """
    List chains
    @return array of chains
    """
    res = self._request("GET", "chain/list")
    XeekAPIException.checkOrRaise(res)
    return APIObjectList(APIChain, res)
    
    
  def createChain(self, name, desc, exploit_ids):
    """
    Add a chain
    @return chain
    """
    exploit_ids = self._jsonencoder.encode(exploit_ids)
    res = self._request("POST", "chain/add", {"name": name, "desc": desc, "exploit_ids": exploit_ids})
    XeekAPIException.checkOrRaise(res)
    return APIChain(res)
  
  
  def getResult(self, result_id):
    """
    Get a result
    @param result_id result id
    @return v
    """
    res = self._request("GET", "result/get", {'id': result_id})
    XeekAPIException.checkOrRaise(res)
    return APIResult(res)
    
  
  def listResults(self, session_id = None, instruction_id = None, audit_id = None):
    """
    List results
    @param session_id Session ID, ignored if None.
    @param instruction_id Instruction ID, ignored if None.
    @param audit_id Audit ID, ignored if None.
    @warning You can specify both session_id and instruction_id, but any other combination of parameters will be rejected.
    @return array of results
    """
    d = {}
    if session_id is not None:
      d["session_id"] = session_id
    if instruction_id is not None:
      d["instruction_id"] = instruction_id
    if audit_id is not None:
      d["audit_id"] = audit_id
      
    res = self._request("GET", "result/list", d)
    XeekAPIException.checkOrRaise(res)
    return APIObjectList(APIResult, res)
    
    
  def deleteResult(self, id):
    """
    Delete a result
    @param id result id
    @return true or false
    """
    res = self._request("POST", "result/delete", {'id': id})
    XeekAPIException.checkOrRaise(res)
    return res
    
    
    
  def listExploits(self):
    """
    List exploits
    @return exploit list
    """
    res = self._request("GET", "exploit/list")
    XeekAPIException.checkOrRaise(res)
    return APIObjectList(APIExploit, res)
  
  
  def getExploit(self, id):
    """
    Get an exploit
    @param id exploit id
    @result exploit
    """
    res = self._request("GET", "exploit/get", {'id': id})
    XeekAPIException.checkOrRaise(res)
    return APIExploit(res)
    
    
  def getConfig(self):
    """
    Get the server configuration
    @result exploit
    """
    res = self._request("GET", "config/get")
    XeekAPIException.checkOrRaise(res)
    return res
    