#!/usr/bin/python
# -*- coding: iso-8859-15 -*-

import traceback
import sys
import os
from inspect import stack

from lib.api import *
from state import *
from banner import *
from parser import *
from token import *
from display_filters import *
from exceptions import *

__version__ = "0.1b"
__author__ = "Emilien \"Trance\" Girault"
__email__ = "trance@segmentationfault.fr | e.girault@sysdream.com"


def usage(usage_string):
  """Decorator used to add a usage string to a method."""
  def wrap(f):
    f.usage = usage_string
    return f
  return wrap


class Console:


  """
  Constructor
  """
  def __init__(self, api_url, debug_mode = True):
  
    self.debug_mode = debug_mode
    
    # Initialize the API and the internal state
    self.api = ClientAPI(api_url)
    try:
      self.state = State(self.api)
    except XeekException, e:
      print e
      sys.exit()
    
    # Parser definition and configuration
    self.parser = Parser()
    
    # Basic commands
    self.parser.addCommand(Token('?',    [], self.help))
    self.parser.addCommand(Token('help', [], self.help, True))
    self.parser.addCommand(Token('quit', [], self.exit))
    self.parser.addCommand(Token('exit', [], self.exit))
    self.parser.addCommand(Token('banner', [], self.banner))
    
    # Show audits, chains, etc.
    self.parser.addCommand(Token('show', [
        Token('audits',         [], self.showAudits),
        #Token('chains',        [], self.showChains),
        Token('queue',          [], self.showQueue),
        Token('exploits',       [], self.showExploits),
        Token('results',        [], self.showResults),
        Token('sessions',       [ Token('online') ], self.showSessions, True),
        Token('params',         [], self.showParams),
        Token('config',         [], self.showConfig),
      ], self.show))
      
    # Get info about an object
    self.parser.addCommand(Token('info', [
        Token('exploit', TokenList(self.exploitIDs, None, []), self.infoExploit, True),
        Token('session', TokenList(self.sessionIDs, None, []), self.infoSession, True),
      ]))
      
    # Use an audit or a chain
    self.parser.addCommand(Token('use', [
        Token('audit',     TokenList(self.auditNames, None, []), self.useAudit, True),
        #Token('chain',   TokenList(self.chainNames, None, []), self.useChain, True),
      ]))
      
    # Go back to the previous mode
    self.parser.addCommand(Token('back', [], self.back))
    
    # Add an exploit to the current audit
    self.parser.addCommand(Token('add', [
        Token('exploit',        TokenList(self.exploitIDs, None, []), self.addExploit, True),
        Token('chain',          TokenList(self.chainNames, None, []), self.addChain, True),
      ]))
    
    # Set a parameter for the current instruction
    self.parser.addCommand(Token('set', TokenList(self.currentParamList, None, []), self.setParam, True))
    
    # Commit
    self.parser.addCommand(Token('commit', [], self.commit))
    
    # Create an audit or a chain
    self.parser.addCommand(Token('create', [
        Token('audit', [], self.createAudit, True),
        Token('chain', [], self.createChain, True),
      ]))
      
    # Delete objects
    self.parser.addCommand(Token('delete', [
        Token('audit',          TokenList(self.auditNames, None, []), self.deleteAudit, True),
        Token('session',        TokenList(self.sessionIDs, None, []), self.deleteSession, True),
        Token('result',         TokenList(self.resultIDs, None, []), self.deleteResult, True),
        Token('exploit',    TokenList(self.instructionIDs, None, []), self.deleteExploit, True),
      ]))
    
    self.parser.addCommand(Token('clean', [
        Token('sessions',     [], self.cleanSessions, True),
        Token('queue',        [], self.cleanQueue, True),
      ]))
    
    
  
  def auditNames(self, unused):
    return [obj['name'] for obj in self.state.getAuditList()]
    
  def chainNames(self, unused):
    return [obj['name'] for obj in self.state.getChainList()]
    
  def exploitIDs(self, unused):
    return [obj['id'] for obj in self.state.getExploitList()]
    
  def sessionIDs(self, unused):
    return [obj['id'] for obj in self.state.getSessionList()]
    
  def resultIDs(self, unused):
    return [obj['id'] for obj in self.state.getResultList()]
    
  def instructionIDs(self, unused):
    return [obj['id'] for obj in self.state.getQueue()]
    
  def currentParamList(self, unused):
    if self.state.instruction_to_add is None:
      return []
    else:
      return map(lambda x: x.upper(), (self.state.instruction_to_add.exploit.params.keys() + ['priority', 'session']))
    
  
  def run(self):
    """Start the console"""
    
    # Print the banner
    self.banner()
    
    print "Xeek Console v%s - Server v%s" % (__version__ , self.state.getConfig()["version"])
    print
    print "Press 'help' to get the list of available commands."
    print

    while 1:
      
      # Display the prompt and handle the command
      try:
        text = raw_input(self.state.prompt)
        self.parser.parseAndExecute(text)
        
      # Handle Ctrl-C
      except KeyboardInterrupt:
        print
        pass
        
      # Handle EOF
      except EOFError:
        break
        
      # Handle XeekExceptions
      except XeekException, e:
        print e
        print
        
      # Handle other exceptions
      except Exception, e:
      
        # If debug mode enabled, print the stack trace
        if(self.debug_mode):
          traceback.print_exc()
        
        # Just print the exception otherwise
        else:
          print e
          print
        
    print
    
    
  def __printTable(self, title, properties, table, indent=2):
    '''
    Print a table containing the results
    @param title title of the table (None if no title)
    @param properties associative array (property API name => name to display)
    @param table array of results
    @param indent number of spaces to print on the left of the table
    '''
  
    # Compute the length of each column to display (except for the last one)
    max_length = {}
    for i in range(len(properties)):
      (prop, name) = properties[i]
      if(i == len(properties)-1):
        max_length[prop] = 0
      else:
        max_length[prop] = max([len(str(obj[prop])) for obj in table] + [len(name)]) + 1
    
    # Print the title if necessary
    if(title):
      print "\n"+title
      print "="*len(title)+"\n"
  
    if(len(table) == 0):
      print " "*indent,
      print "Empty"
      print "\n"
    else:
      # Print the header
      print " "*indent,
      for (prop, name) in properties:
        print name.ljust(max_length[prop]),
        
      print "\n"+" "*indent,
      for (prop, name) in properties:
        print ("-"*len(name)).ljust(max_length[prop]),
      
      # Print the lines
      for obj in table:
        print "\n"+" "*indent,
        for (prop, name) in properties:
          print str(obj[prop]).ljust(max_length[prop]),
      print "\n"
    
  
  def __getArrayParams(self, exploit, actual_params={}):
    params = []
    for name in exploit.params_order:
      props = exploit.params[name]
      myprops = props.copy()
      myprops['name'] = name.upper()
      
      # Required ?
      if 'opt' in myprops:
        if myprops['opt']:
          myprops['required'] = "No"
        else:
          myprops['required'] = "Yes"
        del myprops['opt']
      
      # If the param is set, get its value, else get its default value if it exists
      if name in actual_params:
        myprops['def'] = actual_params[name]
      else:
        if myprops['def'] is None:
          myprops['def'] = ""
      params.append(myprops)
    return params
    
    
  def __askForConfirmation(self, message):
    """Ask the user for confirmation. Return True if the user said 'yes', False otherwise."""
    print
    print message
    text = None
    
    while text != "yes":
      if text is not None:
        print "Invalid answer. Please answer 'yes' or 'no'."
    
      text = raw_input("Do you want to continue ? ")
      if text == "no":
        return False
    
    return True
    
    
  def __showUsage(self):
    """Print usage of the calling method. It must be defined using the @usage decorator."""
    raise XeekException("Usage: "+self.__class__.__dict__[stack()[1][3]].usage)
    
    
  def __updatePrompt(self):
    self.state.updatePrompt()
    
  def switchMode(self, new_mode):
    self.state.switchMode(new_mode)
    
  def backToPreviousMode(self):
    self.state.backToPreviousMode()
      
    
    
  ################################
  # Supported commands
  ################################
  
  def notYetImplemented(self, params):
    print "Method not yet implemented in xeekconsole! Params = " + str(params) + ""
    
    
  def exit(self, unused=None):
    """Exit the program."""
    raise EOFError()
    
  def banner(self, unused=None):
    """Print a banner."""
    print banner()
    
    
  @usage("help [command]")
  def help(self, argv=None):
    """Display general help or help about a specific command."""
    
    # If no arguments, print the list of all commands with description for each one
    if(len(argv) == 1):
      print self.help.__doc__
      
      commands = [{"cmd": c, "desc": self.parser.getHelp(c.split(" "))["doc"]} for c in self.parser.listCommands()]
      self.__printTable("Available commands", [("cmd","Command"), ("desc", "Description")], commands)
      
    # Otherwise, display help for the specified command
    else:
      help = self.parser.getHelp(argv[1:])
      if 'usage' in help:
        print "Usage: "+help["usage"]
      print help["doc"]
      
    print
  
  
  def show(self, unused=None):
    """Show most of Xeek objects."""
    
    if(self.state.selected_audit is None):
      self.showAudits()
      
    self.showExploits()
    #self.showChains()
    self.showSessions()
    
    if(self.state.selected_audit is not None):
      self.showQueue()
      self.showResults()
    
  
  def showExploits(self, unused=None):
    """Show all available exploits."""
    self.__printTable("Exploits", [("id","Id"), ("name", "Name")], self.state.getExploitList())
  
  def showAudits(self, unused=None):
    """Show all audits."""
    list = self.state.getAuditList()
    
    # Add some properties to each audit
    for a in list:
      a["nb_sessions"] = len(self.api.listSessions(a.id))
      a["nb_results"] = len(self.api.listResults(None, None, a.id))
      
    self.__printTable("Audits", 
                      [("name","Name"), 
                      ("nb_sessions", "# Sessions"), 
                      ("nb_results", "# Results"), 
                      ("token", "Token"), 
                      ("created_at", "Created at")], 
                      list)
    
  def showChains(self, unused=None):
    """Show all exploit chains."""
    self.__printTable("Chains", [("name","Name"), ("description","Description")], self.state.getChainList())
    
  
  @usage("show sessions [online]")
  def showSessions(self, argv=[]):
    """Show sessions for the current audit."""
    
    online = (len(argv) == 3 and argv[2] == "online")
    
    sessions = self.state.getSessionListFromSelectedAudit(online)
    
    titles = [("id", "Id")]
    
    if(self.state.selected_audit is None):
      titles.append(("audit_name", "Audit"))
      audits = self.state.getAuditList()
      audit_names = {}
      for a in audits:
        audit_names[a.id] = a.name
      for i in range(len(sessions)):
        sessions[i].audit_name = audit_names[sessions[i].audit_id]
    
    
    titles += [("ip","IP"), 
              ("browser","Browser"), 
              ("os", "OS"), 
              ("created_at", "Created at"),
              ("last_seen", "Last seen at")]
      
    self.__printTable("Sessions", 
                      titles, 
                      map(lambda r: useragentfilter(r), sessions))
    
    
  def showResults(self, unused=None):
    """Show results of the current audit."""
    if(self.state.current_mode == MODE_SELECT_AUDIT):
      results = self.state.getResultListFromSelectedAudit()
      self.__printTable("Results", 
                        [("id", "ID"), 
                        ("session_id","Sess. ID"), 
                        ("instruction_id", "Inst. ID"), 
                        ("exploit_id", "Exploit"), 
                        ("created_at", "Created at"),
                        ("data_string_list", "Data")], 
                        results)
    
    else:
      raise XeekException("You must use the 'use audit' command first.")
      
   
  def showQueue(self, unused=None):
    """Show the queue of the current audit."""
    if(self.state.current_mode != MODE_SELECT_AUDIT):
      raise XeekException("You must use the 'use audit' command first.") # TODO On devrait plutot tester l'audit slectionn, pour pouvoir utiliser la commande dans un sous-mode...
    self.__printTable("Queue", 
                      [("id","Id"), 
                      ("exploit_id","Exploit"), 
                      ("prio","Prio"), 
                      ("created_at", "Created at"),
                      ("params", "Parameters")], 
                      self.api.getQueue(self.state.selected_audit.id))
    
    
  def showParams(self, unused=None):
    """Show exploit parameters."""
    if(self.state.current_mode != MODE_ADD_INST):
      raise XeekException("You must use the 'add instruction' command first.")
    
    self.__printTable("Parameters", 
                      [("name", "Name"), 
                      ("required", "Required"), 
                      ("def", "Value"), 
                      ("desc", "Description")], 
                      self.__getArrayParams(self.state.instruction_to_add.exploit, 
                                            self.state.instruction_to_add.params))
                                            
                                            
  def showConfig(self, unused=None):
    """Show server configuration variables."""
    config_values = self.state.getConfig()
    config_labels = ["version", "rpc_url", "fetch_interval_secs"]
    max_label_length = max([len(k) for k in config_labels]) + 1
    
    print
    print "Configuration variables:"
    print "========================"
    print
    for name in config_labels:
      print name.upper().rjust(max_label_length) + ": " + config_values[name]    
    print 
    
  
  @usage("info exploit EXPLOIT_ID")
  def infoExploit(self, argv):
    """Show information about an exploit."""
    if(len(argv) != 3):
      self.__showUsage()
     
    try:
      exploit = self.api.getExploit(argv[2])
    except Exception, e:
      raise XeekError("Invalid exploit id.")
      
    authors_text = "Author"
    if(len(exploit.authors) > 1):
      authors_text += "s"
          
    infos = [("Name",           exploit.name),
             ("Version",        exploit.version),
             ("Description",    exploit.description),
             (authors_text,     ", ".join(exploit.authors))]    
    max_label_length = max([len(k) for k,d in infos]) + 1
    
    print
    for label, desc in infos:
      print label.rjust(max_label_length) + ": " + desc
    
    print
    print "Parameters:"
    self.__printTable(None, 
                      [("name", "Name"), ("required", "Required"), ("def", "Default Value"), ("desc", "Description")], 
                      self.__getArrayParams(exploit))
     
    
  @usage("info session SESSION_ID")
  def infoSession(self, argv):
    """Show information about a session."""
    if(len(argv) != 3):
      self.__showUsage()
      
    try:
      session = self.api.getSession(argv[2])
    except Exception, e:
      raise XeekError("Invalid session id.")
      
      
    infos = [("Id",                         session.id),
             ("Current instruction",        session.instruction_id),
             ("Victim's IP",                session.ip),
             ("User-agent",                 session.useragent),
             ("Created at",                 session.created_at),
             ("Last seen at",               session.last_seen),
             ]    
    max_label_length = max([len(k) for k,d in infos]) + 1
    
    print
    for label, desc in infos:
      print label.rjust(max_label_length) + ": " + desc
    
    print
      
    
  @usage("use audit AUDIT_NAME")
  def useAudit(self, argv):
    """Select an audit."""
    if(len(argv) != 3):
      self.__showUsage()
      
    audit = self.state.getAudit(argv[2])
    if(audit is None):
      raise XeekError("Invalid audit name.")
      
    self.state.selected_audit = audit
    self.switchMode(MODE_SELECT_AUDIT)
    
    
  @usage("use chain CHAIN_NAME")
  def useChain(self, argv):
    """Select an exploit chain."""
    if(len(argv) != 3):
      self.__showUsage()
      
    chain = self.state.getChain(argv[2])
    if(chain is None):
      raise XeekError("Invalid chain name.")
      
    self.state.selected_chain = chain
    self.switchMode(MODE_SELECT_CHAIN)
    
    
  @usage("add exploit INSTRUCTION_NAME")
  def addExploit(self, argv):
    """Add an exploit to the queue of the current audit."""
    if(self.state.current_mode != MODE_SELECT_AUDIT):
      raise XeekException("You must use the 'use audit' command first.")
      
    if(len(argv) != 3):
      self.__showUsage()
      
    try:
      exploit = self.api.getExploit(argv[2])
    except: 
      raise XeekError("Invalid exploit name.")
    
    self.state.instruction_to_add = Instruction(exploit)
    self.switchMode(MODE_ADD_INST)
    print "EXPLOIT => " + argv[2]
    
    
  @usage("set PARAM VALUE")
  def setParam(self, argv):
    """Set a parameter to the current exploit. Valid params include those listed with 'show params' as well as SESSION and PRIORITY."""
    if(self.state.current_mode != MODE_ADD_INST):
      raise XeekException("You must use the 'add exploit' command first.")
      
    if(len(argv) != 3):
      self.__showUsage()
      
    name = argv[1].lower()
    val = argv[2]
    
    # Handle the 'set session' command
    if name == "session":
    
      # Check the session ID
      try:
        self.api.getSession(val)
      except:
        raise XeekError("Invalid session ID")
      
      self.state.instruction_to_add.session_id = val
      print "SESSION => " + val
      
    # Handle the 'set priority' command
    elif name == "priority":
    
      # Check the priority
      try:
        prio = int(val)
      except:
        raise XeekError("Invalid priority; must be an integer")
      
      if prio <= 0:
        raise XeekError("Invalid priority; must be > 0")
        
      self.state.instruction_to_add.prio = prio
      print "PRIORITY => " + val
      
    # Handle the 'set PARAM' command
    elif name in self.state.instruction_to_add.exploit.params:
      self.state.instruction_to_add.params[name] = val
      print name.upper() + " => " + val
      
    else:
      raise XeekError("Invalid parameter.\nYou can use parameters listed using 'show params', as well as SESSION and PRIORITY options.")
    
    
  def commit(self, argv):
    """Commit the current operation."""
    if(self.state.current_mode == MODE_ADD_INST):
      
      # Check parameters
      # If there is at least one missing parameter that is not optional and has no default value, raise an exception
      for name, props in self.state.instruction_to_add.exploit.params.items():
        if not name in self.state.instruction_to_add.params and not props['opt'] and props['def'] is None:
          raise XeekException("Parameter '"+name.upper()+"' is missing.")
      
      self.state.pushCurrentInstructionToQueue()
      print "Exploit added to audit queue."
      self.backToPreviousMode()
    
    else:
      raise XeekException("You must use the 'add' command first.")
    
    
  @usage("add chain CHAIN_NAME")
  def addChain(self, argv):
    """Add a chain."""
    if(self.state.current_mode != MODE_SELECT_AUDIT):
      raise XeekException("You must use the 'use audit' command first.")
      
    if(len(argv) != 3):
      self.__showUsage()
      
    chain = self.state.getChain(argv[2])
    if(chain is None):
      raise XeekError("Invalid chain name.")
      
    self.state.chain_to_add = chain
    self.switchMode(MODE_ADD_CHAIN)
    print "NAME => " + argv[2]
    
    
  def back(self, unused=None):
    """Go back to the previous mode."""
    
    # Reset attributes and switch mode
    if(self.state.current_mode == MODE_SELECT_AUDIT):
      self.state.selected_audit = None
    
    self.backToPreviousMode()
  
  
  @usage("create audit AUDIT_NAME")
  def createAudit(self, argv):
    """Create an audit."""
    if(len(argv) != 3):
      self.__showUsage()
  
    try:
      audit = self.api.createAudit(argv[2])
    except InvalidParameter, e:
      raise XeekError("There already is an audit with the selected name. Please choose another one.")
    
    print "Created audit '"+argv[2]+"'."
    print "Exploit URL: "+ self.state.getURLAudit(audit)
    
    
  @usage("Usage: create chain CHAIN_NAME")
  def createChain(self, argv):  
    """Create a chain."""
    if(len(argv) != 3):
      self.__showUsage()
    
    self.state.chain_to_add = Chain(argv[2])
    print "NAME => " + argv[2]
    self.switchMode(MODE_CREATE_CHAIN)
    
    
  @usage("delete audit AUDIT_NAME")
  def deleteAudit(self, argv):
    """Delete an audit."""
    if(self.state.current_mode != MODE_EXT):
      raise XeekException("You must not be inside an audit to use this command. Use the 'back' command.")
      
    if(len(argv) != 3):
      self.__showUsage()
      
    audit = self.state.getAudit(argv[2])
    if(audit is None):
      raise XeekError("Invalid audit name.")
      
    # Ask for confirmation
    if(self.__askForConfirmation("You are about to delete audit "+audit.name+"!")):
      self.api.deleteAudit(audit.id)
      print "Audit "+audit.name+" has beed successfully deleted."
    
  
  @usage("delete session SESSION_ID")
  def deleteSession(self, argv):
    """Delete a session."""
    if(len(argv) != 3):
      self.__showUsage()
      
    try:
      self.api.getSession(argv[2])
    except:
      raise XeekError("Invalid session id.")
      
    self.api.deleteSession(argv[2])
    print "Session "+argv[2]+" has beed successfully deleted."
    
    
  @usage("delete result RESULT_ID")
  def deleteResult(self, argv):
    """Delete a result."""
    if(len(argv) != 3):
      self.__showUsage()
      
    try:
      self.api.getResult(argv[2])
    except:
      raise XeekError("Invalid result id.")
      
    self.api.deleteResult(argv[2])
    print "Result "+argv[2]+" has beed successfully deleted."
  
  
  @usage("delete exploit INSTRUCTION_ID")
  def deleteExploit(self, argv):
    """Delete an exploit from the current queue."""
    if(len(argv) != 3):
      self.__showUsage()
    
    if(self.state.current_mode != MODE_SELECT_AUDIT):
      raise XeekException("You must use the 'use audit' command first.")
      
    if not self.api.deleteInstruction(argv[2]):
      raise XeekError("Invalid exploit id.")
      
    print "Exploit "+argv[2]+" has beed successfully deleted."
  
  
  def cleanSessions(self, argv):
    """Delete all sessions from the current audit, or from all audits if no one is selected. Use 'force' to say 'yes' automatically."""
      
    if(self.state.current_mode != MODE_SELECT_AUDIT and self.state.current_mode != MODE_EXT):
      raise XeekException("You cannot use this command in this mode. Please use the 'back' command first.")
      
    audit_id = 0
    from_msg = "all audits"
    if(self.state.current_mode == MODE_SELECT_AUDIT):
      audit_id = self.state.selected_audit.id
      from_msg = "audit " + self.state.selected_audit.name
    
    res = (len(argv) == 3 and argv[2] != "force") or self.__askForConfirmation("You are about to delete all sessions for "+from_msg+"!")
    if(res):
      self.api.cleanSessions(audit_id)
      print "All instructions deleted for "+from_msg+"."
      
  
  def cleanQueue(self, argv):
    """Delete the queue for the current audit. Use 'force' to say 'yes' automatically."""
      
    if(self.state.current_mode != MODE_SELECT_AUDIT):
      raise XeekException("You must use the 'use audit' command first.")
      
    audit_id = self.state.selected_audit.id
    from_msg = "audit " + self.state.selected_audit.name
    
    res = (len(argv) == 3 and argv[2] != "force") or self.__askForConfirmation("You are about to delete all instructions for "+from_msg+"!")
    if(res):
      self.api.cleanQueue(audit_id)
      print "All sessions deleted for "+from_msg+"."
  
def print_help():
  """Print help"""
  
  print
  print "    XeekConsole v%s" % (__version__)
  print "    by %s <%s>" % (__author__, __email__)
  print
  print "Usage: %s server_url " % os.path.basename(sys.argv[0])
  print
  print "   server_url: URL used to dialog with Xeek server"
   
  
def run():
  """Main routine"""
  
  if(len(sys.argv) != 2 or sys.argv[1] == "-h" or sys.argv[1] == "--help"):
    print_help()
    sys.exit(0)
  
  c = Console(sys.argv[1])
  c.run()
