<?php

require_once(dirname(__FILE__)."/ApiException.class.php");

/**
 * @class CoreApi
 * @brief Core of the API. Implements the Singleton desing pattern.
 */
class CoreApi {

  //Singleton instance
  protected static $s_instance = null;
  
  //Reference to the core
  protected $core;
  
  //Array of methods that are allowed to be called from the outside
  protected static $api_methods = array("audit/add"                => "createAudit",
                                        "audit/get"                => "getAudit",
                                        "audit/list"               => "listAudits",
                                        "audit/delete"             => "deleteAudit",
                                        
                                        //Renommer en instruction/list ?
                                        "queue/get"                => "getQueue",
                                        "queue/instruction/add"    => "pushInstructionToQueue",
                                        "queue/instruction/delete" => "deleteInstruction",
                                        //"queue/chain/add"          => "pushChainToQueue",
                                        "queue/clean"              => "cleanQueue",
                                        
                                        "session/list"             => "listSessions",
                                        "session/get"              => "getSession",
                                        "session/reset"            => "resetSession",
                                        "session/delete"           => "deleteSession",
                                        "session/clean"            => "cleanSessions",
                                        
                                        //"chain/list"               => "listChains",
                                        //"chain/get"                => "getChain",
                                        //"chain/add"                => "addChain",
                                        
                                        //rajouter des mthodes chain/instruction/add, chain/instruction/delete ?
                                        
                                        //"chain/delete"             => "notYetImplemented",    //TODO
                                        
                                        "result/list"              => "listResults",
                                        "result/get"               => "getResult",
                                        "result/delete"            => "deleteResult",
                                        
                                        "exploit/list"             => "listExploits",
                                        "exploit/get"              => "getExploit",
                                        
                                        "config/get"               => "getConfig",
                                        
                                        );
  
  
  /**
   * @brief Private constructor. Use getInstance() method instead.
   */
  private function __construct(){
    $this->core = Core::getInstance();
    $this->method_props = $this->buildMethodProps();
  }
  
  /**
   * @brief Get the unique instance of the Core class
   * @return unique instance
   */
  public static function getInstance() {
    if(!self::$s_instance){
      self::$s_instance = new CoreApi();
    }
    return self::$s_instance;
  }
  
  /**
   * @brief Build the method properties array using reflexion
   * @return array (external name => ('name' => internal name, 'params' => (name => (index, isOptional, default value) ) ) )
   */
  protected function buildMethodProps(){
    $a = array();
    $c = new ReflectionClass("CoreApi");
    
    //For each method
    foreach(self::$api_methods as $ext_name => $int_name) {
    
      //Get its parameters using reflexion
      $params_ref = $c->getMethod($int_name)->getParameters();
      $p = array();
    
      //For each parameter, build an array that contains the name and the index
      $i = 0;
      foreach($params_ref as $p_ref){
        $isOpt = (bool) $p_ref->isOptional();
        $p[$p_ref->getName()] = array($i++, $isOpt, $isOpt ? $p_ref->getDefaultValue() : null);
      }
    
      $a[$ext_name] = array("name" => $int_name, "params" => $p);
    }
    
    return $a;
  }
  
  /**
   * @brief Decode a json array if necessary
   * @param $params array
   * @return array or null if it cannot be decoded
   */
  protected function decodeArray($params){
    if(!is_array($params)) {
      $params = Core::getInstance()->json->decode($params);
    }
    return is_array($params) ? $params: null;
  }
  
  
  /**
   * @brief Call an API method and check both method name and parameters.
   * @param $method_ext_name external name of the method (in the x/y form)
   * @param $params array of parameters (name => value)
   * @return Return value of the called method
   */
  public function callMethod($method_ext_name, $params) {
  
    //Lookup the real method name
    if(!array_key_exists($method_ext_name, $this->method_props)){
      throw new InvalidMethod();
    }
    $method = $this->method_props[$method_ext_name];
    $method_int_name = $method['name'];
    
    //Check params and build param array
    $final_params = array();
    foreach($method['params'] as $name => $arr){
    
      //If the parameter is provided
      if(array_key_exists($name, $params)){
        $final_params[$arr[0]] = $params[$name];
      } else{ //If it is not provided
        if(!$arr[1]) { //If it is not optional => error
          throw new MissingParameter($name);
        } else { //If it is optional => take its default value
          $final_params[$arr[0]] = $arr[2];
        }
      }
    }
    
    //Call the method and return the result
    return call_user_func_array(array($this, $method_int_name), $final_params);
  }
  
  
  ##############################################################
  # API Methods
  ##############################################################
  
  
  /**
   * @brief Create an audit
   * @param $name name of the audit
   * @return associative array representing the new audit (id, name, token)
   */
  public function createAudit($name) {
    //Check that the name does not already exist
    if($this->core->getAuditByName($name) != null) {
      throw new InvalidParameter("name");
    }
    $audit = $this->core->createAudit($name);
    return $audit->serialize();
  }

  /**
   * @brief Get an audit
   * @param $id id of the audit
   * @return associative array representing the audit (id, name, token)
   */
  public function getAudit($id) {
    $audit = $this->core->getAudit($id);
    if($audit != null){
      return $audit->serialize();
    } else {
      throw new InvalidParameter("id");
    }
  }
  
  /**
   * @brief List all audits
   * @return list of associative arrays representing the audit (id, name, token)
   */
  public function listAudits() {
    $audits = $this->core->listAudits();
    $res = array();
    foreach($audits as $id => $a){
      $res[] = $a->serialize();
    }
    return $res;
  }
  
  /**
   * @brief Delete an audit
   * @param $id audit id
   * @return true if everything goes well, false otherwise.
   */
  public function deleteAudit($id) {
    return $this->core->deleteAudit($id);
  }
  
  /**
   * @brief Get the queue of an audit
   * @return audit or throw an exception 
   */
  public function getQueue($audit_id) {
    try {
      return $this->core->getQueue($audit_id)->serialize();    
    } catch(Exception $e){
      print_r($e);
      throw new InvalidParameter("audit_id");
    }
  }
  
  /** 
   * @brief Push an instruction to the queue
   * @param $audit_id
   * @param $exploit_id
   * @param $params
   * @param $session_id
   * @param $prio
   * @result instruction
   */
  public function pushInstructionToQueue($audit_id, $exploit_id, $params, $session_id = 0, $prio = Instruction::PRIO_DEFAULT) {
    try {
      //Decode the $params parameter if necessary
      if(!is_array($params)){
        $params = Core::getInstance()->json->decode($params);
      }
      return $this->core->pushInstructionToQueue($audit_id, $exploit_id, $params, $session_id, $prio)->serialize(); 
    } catch(InvalidAudit $e){
      throw new InvalidParameter("audit_id");
    } catch(InvalidSession $e){
      throw new InvalidParameter("session_id");
    } catch(InvalidExploit $e){
      print_r($e);
      throw new InvalidParameter("exploit_id");   
    } catch(MissingExploitParameter $e){
      throw new InvalidParameter("params");
    }
  }
  
  
  /** 
   * @brief Push a chain to the queue
   * @param $audit_id
   * @param $chain_id
   * @param $params
   * @param $prio
   * @result instruction
   */
  public function pushChainToQueue($audit_id, $chain_id, $params, $prio = Instruction::PRIO_DEFAULT) {
    if(!($params = $this->decodeArray($params))){
      throw new InvalidParameter("params");
    }
    try {
      //Decode the $params parameter if necessary
      $instructions =  $this->core->pushChainToQueue($audit_id, $chain_id, $params, $prio);
      $res = array();
      foreach($instructions as $a){
        $res[] = $a->serialize();
      }
      return $res;
      
    } catch(InvalidAudit $e){
      throw new InvalidParameter("audit_id");
    } catch(InvalidChain $e){
      throw new InvalidParameter("chain_id");
    } catch(IncorrectNumberOfParameters $e){
      throw new InvalidParameter("params");
    } catch(MissingExploitParameter $e){
      throw new InvalidParameter("params");
    }
  }
  
  /**
   * @brief Delete an instruction
   * @param $id instruction id
   * @return true if everything goes well, false otherwise.
   */
  public function deleteInstruction($id) {
    return $this->core->deleteInstruction($id);
  }
  
  /**
   * @brief Delete all instructions in an audit queue
   * @param $audit_id Audit id to clean
   */
  public function cleanQueue($audit_id) {
    try {
      return $this->core->cleanQueue($audit_id);
    } catch(InvalidAudit $e) {
      throw new InvalidParameter("audit_id");
    }
  }
  
  
  /**
   * @brief List all sessions for all audit, or for a perticular one
   * @param $audit_id audit id
   * @param $online 1 to get only online sessions, 0 otherwise
   * @return array of sessions
   */
  public function listSessions($audit_id = 0, $online = 0){
    $sessions = $this->core->listSessionsByAudit($audit_id, $online);
    $a = array();
    foreach($sessions as $s){
      $a[] = $s->serialize();
    }
    return $a;
  }
  
  /**
   * @brief Get a session
   * @param $id session id
   * @return session
   */
  public function getSession($id){
    $s = $this->core->getSession($id);
    if($s != null){
      return $s->serialize();
    } else {
      throw new InvalidParameter("id");
    }
  }
  
  /**
   * @brief Reset the instruction_id of a session
   * @param $id Session ID
   * @param $instruction_id Instruction ID, 0 by default
   * @return updated session
   */
  public function resetSession($id, $instruction_id = 0){
    $s = $this->core->resetSession($id, $instruction_id);
    if($s != null){
      return $s->serialize();
    } else {
      throw new InvalidParameter("id");
    }
  }
  
  
  /**
   * @brief Delete a session
   * @param $id Session ID
   * @return true if everything goes well, false otherwise
   */
  public function deleteSession($id){
    return $this->core->deleteSession($id);
  }
  
  /**
   * @brief Delete all sessions
   * @param $audit_id audit id to clean. If not provided, delete all sessions.
   */
  public function cleanSessions($audit_id = 0) {
    return $this->core->cleanAllSessions($audit_id);
  }
  
  
  /**
   * @brief Get a chain
   * @param $id chain id
   * @return chain or throw an exception 
   */
  public function getChain($id){
    $c = $this->core->getChain($id);
    if($c != null){
      return $c->serialize();
    } else {
      throw new InvalidParameter("id");
    }
  }
  
  /**
   * @brief List all chains
   * @return chains
   */
  public function listChains(){
    $chains = $this->core->listChains();
    $a = array();
    foreach($chains as $c){
      $a[] = $c->serialize();
    }
    return $a;
  }
  
  
  /** 
   * @brief Add a chain
   * @param $name name of the chain
   * @param $desc description
   * @param $exploit_ids array of exploit names, empty by default
   * @return new chain or throw an exception if the exploit names are incorrect
   */
  public function addChain($name, $desc, $exploit_ids = array()){
    if(!($exploit_ids = $this->decodeArray($exploit_ids))){
      new InvalidParameter("exploit_ids");
    }
    try {
      return $this->core->addChain($name, $desc, $exploit_ids)->serialize();
    } catch(InvalidExploit $e) {
      throw new InvalidParameter("exploit_ids");
    }
  }
  
  
  /**
   * @brief Get a result
   * @param $id result id
   * @return result
   */
  public function getResult($id){
    $s = $this->core->getResult($id);
    if($s != null){
      return $s->serialize();
    } else {
      throw new InvalidParameter("id");
    }
  }
  
  /**
   * @brief List all results for a given session, instruction, or audit
   * @param $session_id session id. Ignored if null.
   * @param $instruction_id instruction id. Ignored if null.
   * @param $audit_id audit id. Ignored if null.
   * @warning You can provide both session_id and instruction_id but any combination using audit_id is forbidden.
   * @return array of sessions
   */
  public function listResults($session_id = null, $instruction_id = null, $audit_id = null){
  
    //Check parameters and use the appropriate method
    if($audit_id == null){
      $results = Result::listBySessionAndInstruction($session_id, $instruction_id); //TODO call the Core instead
    } else {
      if($session_id != null || $instruction_id != null){
        throw new InvalidParameterCombination();
      }
      $results = Result::listByAudit($audit_id);                                    //TODO call the Core instead
    }
    
    $a = array();
    foreach($results as $r){
      $a[] = $r->serialize();
    }
    return $a;
  }
  
  
  
  /**
   * @brief Delete a result
   * @param $id result id
   * @return true if everything goes well, false otherwise
   */
  public function deleteResult($id){
    return $this->core->deleteResult($id);
  }
  
  
  /** 
   * @brief List all registered exploits
   * @return exploit class list
   */
  public function listExploits(){
    $l = $this->core->listExploits();
    $a = array();
    foreach($l as $e){
      $a[] = $e->serialize();
    }
    return $a;
  }
  
  
  /** 
   * @brief Get an exploit
   * @param $id id of the exploit
   * @return exploit
   */
  public function getExploit($id){
    $e = $this->core->getExploit($id);
    if($e != null){
      return $e->serialize();
    } else {
      throw new InvalidParameter("id");
    }
  }
  
  
  /**
   * @brief Get the configuration of the server
   * @return configuration
   */
  public function getConfig(){
    return $this->core->getConfig()->serialize();
  }
  
  
  /**
   * @brief Default behaviour for methods that are not yet implemented
   */
  public function notYetImplemented(){
    throw new NotYetImplemented();
  }
  
}

?>