<?php

require_once(dirname(__FILE__)."/../conf/general.php");

/**
 * @class Core
 * @brief Core class. Implements Mediator and Singleton design patterns.
 */
class Core {

  //Singleton instance
  protected static $s_instance = null;
  
  //Audits
  protected $audits;
  
  //JSON service
  public $json;
  
  
  /**
   * @brief Constructor
   *        Don't call it directly! Use static method getInstance() instead.
   */
  private function __construct(){
  
    //Register exploits
    $this->registerExploits();
  
    //Build the JSON (en|de)coder
    $this->json = new JsonFormat();
    
    //Fetch audits
    $audits = Audit::fetchList();
    $this->audits = array();
    foreach($audits as $a){
      $this->audits[$a->id] = $a;
    }
  }
  
  /**
   * @brief Get the unique instance of the Core class
   * @return unique instance
   */
  public static function getInstance() {
    if(!self::$s_instance){
      self::$s_instance = new Core();
    }
    return self::$s_instance;
  }
  
  /**
   * @brief Debug string
   */
  public function __toString() {
    return "Core is working!";
  }
  
  /**
   * @brief Create an audit
   * @param $name name of the audit
   * @return new audit
   */
  public function createAudit($name) {
    $audit = new Audit(array("name" => $name));
    $audit->save();
    $this->audits[$audit->id] = $audit;
    return $audit;
  }
  
  /**
   * @brief List all audits
   * @return audit list
   */
  public function listAudits() {
    return $this->audits;
  }
  
  /**
   * @brief Get an audit using its id
   * @param $i audit id
   * @return audit or null if no audit corresponds
   */
  public function getAudit($i) {
    if(isset($this->audits[$i])) {
      return $this->audits[$i];
    } else {
      return null;
    }
  }
  
  /**
   * @brief Get an audit using its name
   * @param $name audit name
   * @return audit or null if no audit corresponds
   */
  public function getAuditByName($name) {
    foreach($this->audits as $a) {
      if($a->name == $name) {
        return $a;
      }
    }
    return null;
  }
  
  /**
   * @brief Delete an audit
   * @param $i audit id
   * @return true or false
   */
  public function deleteAudit($i) {
    $a = $this->getAudit($i);
    if($a != null){
      $a->delete();
      return true;
    } else {
      return false;
    }
  }  
  
  
  /**
   * @brief Get the queue of an audit
   * @return audit or throw an exception if no audit correspond no audit corresponds
   */
  public function getQueue($audit_id) {
    $a = $this->getAudit($audit_id);
    if($a == null){
      throw new InvalidAudit();
    }
    return $a->queue;
  }
  
  /**
   * @brief Get an instruction
   * @param $id instruction id
   * @return instruction
   */
  public function getInstruction($id){
    return Instruction::fetchById($id);
  }
  
  /**
   * @brief Push an instruction onto the queue of an audit
   * @param $audit_id audit id
   * @param $exploit_id exploit name
   * @param $params
   * @param $session_id
   * @param $prio
   * @return instruction
   */
  public function pushInstructionToQueue($audit_id, $exploit_id, $params, $session_id = 0, $prio = Instruction::PRIO_DEFAULT) {
    $q = $this->getQueue($audit_id);
    $i = $q->pushInstr($exploit_id, $params, $session_id, $prio);
    return $i;
  }
  
  /**
   * @brief Push an instruction onto the queue of an audit
   * @param $audit_id audit id
   * @param $chain_id chain id
   * @param $params
   * @param $prio
   * @return queue or throw an exception if one of the parameters is incorrect
   */
  public function pushChainToQueue($audit_id, $chain_id, $params, $prio = Instruction::PRIO_DEFAULT){
    $q = $this->getQueue($audit_id);
    if($q == null) throw new InvalidAudit();
    $c = Chain::fetchById($chain_id);
    if($c == null) throw new InvalidChain();
    return $q->pushChain($c, $params, $prio);
  }
  
  
  /**
   * @brief Delete an instruction from the queue
   * @param $id instruction id
   * @return true or false
   */
  public function deleteInstruction($id) {
    $i = $this->getInstruction($id);
    if($i != null){
      $i->delete();
      return true;
    } else {
      return false;
    }
  } 
  
  /**
   * @brief Delete all instructions in an audit queue
   * @param $audit_id Audit id to clean
   */
  public function cleanQueue($audit_id) {
    $q = $this->getQueue($audit_id);
    if($q == null) throw new InvalidAudit();
    return $q->clean();
  } 
  
  /**
   * @brief List all sessions
   * @return array of sessions
   */
  public function listSessions(){
    return Session::listAll();
  }
  
  /**
   * @brief List all sessions for a given audit
   * @param $audit_id audit id
   * @param $online true to get only online sessions
   * @return array of sessions
   */
  public function listSessionsByAudit($audit_id, $online = false){
    return Session::listSessions($audit_id, $online);
  }
  
  
  /**
   * @brief Get a session
   * @param $id session id
   * @return session
   */
  public function getSession($id){
    return Session::fetchById($id);
  }
  
  
  /**
   * @brief Delete a session
   * @param $i session id
   * @return true or false
   */
  public function deleteSession($i) {
    $s = $this->getSession($i);
    if($s != null){
      $s->delete();
      return true;
    } else {
      return false;
    }
  }
  
  /**
   * @brief Delete all sessions
   * @param $audit_id audit id to clean. If not provided, delete all sessions.
   */
  public static function cleanAllSessions($audit_id = 0) {
    return Session::cleanAll($audit_id);
  }
  
  
  /**
   * @brief Get a chain
   * @param $id chain id
   * @return chain
   */
  public function getChain($id){
    return Chain::fetchById($id);
  }
  
  /**
   * @brief List all chains
   * @return chains
   */
  public function listChains(){
    return Chain::fetchList();
  }
  
  /** 
   * @brief Add a chain
   * @param $name name of the chain
   * @param $desc description
   * @param $exploit_ids array of exploit names, empty by default
   * @return new chain or throw an exception if the exploit names are incorrect
   */
  public function addChain($name, $desc, $exploit_ids = array()){
  
    //Ensure the exploit names are OK
    foreach($exploit_ids as $e){
      if(Exploit::getExploit($e) == null){
        throw new InvalidExploit();
      }
    }
    
    //Create the chain
    $c = new Chain(array("name" => $name, "description" => $desc));
    $c->save();
    
    //Add exploits
    foreach($exploit_ids as $e){
      $c->addexploit($e);
    }
    
    return $c;
  }
  
  
  
  /**
   * @brief Reset the instruction_id of a session
   * @param $session_id Session ID
   * @param $instruction_id Instruction ID 0 by default
   * @return updated session, or null if the provided $session_id is incorrect
   */
  public function resetSession($session_id, $instruction_id = 0){
    $s = Session::fetchById($session_id);
    if($s != null){
      $s->reset($instruction_id);
      return $s;
    } else {
      return null;
    }
  
  }
  
  /**
   * @brief Get a result
   * @param $id result id
   * @return result
   */
  public function getResult($id){
    return Result::fetchById($id);
  }
  
  
  /**
   * @brief Delete a result
   * @param $i result id
   * @return true or false
   */
  public function deleteResult($i) {
    $r = $this->getResult($i);
    if($r != null){
      $r->delete();
      return true;
    } else {
      return false;
    }
  }
  
  /**
   * @brief Get the configuration of the server
   * @return configuration
   */
  public function getConfig(){
    return Config::getInstance();
  }
  
  
  /** 
   * @brief List all registered exploits
   * @return exploit class list
   */
  public function listExploits(){
    return Exploit::getExploitList();
  }
  
  /** 
   * @brief Get an exploit
   * @param $id id of the exploit
   * @return exploit, null if the id is incorrect
   */
  public function getExploit($id){
    return Exploit::getExploit($id);
  }
  
  
  /**
   * @brief Get XeekAgent code
   * @param $sessid_cookie_name cookie name
   * @param $sessid_cookie_value cookie value
   */
  public function getXeekAgentCode($sessid_cookie_name, $sessid_cookie_value){
  
    //Read XeekAgent code
    $code = file_get_contents("../js/xeekagent.js");
    
    //Set configuration constants
    $config = $this->getConfig();
    $code = preg_replace("#XEEK_RPC_URL#", '"'.$config->rpc_url.'"', $code);
    $code = preg_replace("#XEEK_FETCH_INTERVAL_SECS#", $config->fetch_interval_secs, $code);
    $code = preg_replace("#XEEK_SESSID_COOKIE#", '"'.$sessid_cookie_name."=".$sessid_cookie_value.'"', $code);
    
    
    //TODO remove comments
    
    return $code;
  }
  
  /**
   * @brief register all exploits
   */
  protected function registerExploits($dir = "") {
  
    $realdir = dirname(__FILE__)."/../modules/exploits".$dir;
    if($exploits_dir = @opendir($realdir)){
    
      //Scan the exploit folder
      while($file = readdir($exploits_dir)){
        if(!preg_match("/^\./", $file)) {
        
          //Register .js files as exploits
          if(preg_match("/\.js$/", $file)) {
            Exploit::register(ltrim($dir."/".$file,"/"));
          }
          
          //Scan folders recursively
          if(is_dir($realdir."/".$file)){
            $this->registerExploits("/".$file);
          }
        }
      }
      
      closedir($exploits_dir);
    
    } else {
      throw new InternalException("Core::registerExploits: Unable to open modules/exploits directory.");
    }
  
  
  }
}

?>
