<?php

/**
 * @class Exploit
 * @brief Represent an exploit
 */
class Exploit extends Module {
  
  //List of all registered exploits: array ( class name => ( JS files ) )
  protected static $exploit_list = array();
  
  //JS file path of the exploit
  protected $js_file_path;
  
  //Parameters description
  protected $param_desc;
  
  //Parameters order
  protected $param_order;
  protected $param_current;

  
  
  /** 
   * @brief Protected contructor
   * @param $js_file_path path to the JS file implementing the exploit
   */
  protected function __construct($js_file_path){
  
    if(preg_match("#(.+)\.js$#", $js_file_path, $res)){
      $this->js_file_path = $js_file_path;
      $this->id = $res[1];
    } else {
      throw new InternalException("Exploit::__construct(): The specified file is not a .js file.");
    }
    
    $this->param_desc = array();
    $this->param_order = array();
    $this->param_current = 0;
  
    parent::__construct($this->js_file_path);
    $this->type = Module::TYPE_EXPLOIT;
  }
  
  
  /**
   * @brief Read the header of an exploit file
   * @param $exploit_path module path
   * @return header of empty string if no header was found
   */
  protected function readHeader($exploit_path){
    return parent::readHeader("/exploits/".$exploit_path);
  }
  
  /**
   * @brief Parse the header of the JS file
   * @param $header exploit header
   * @return property array
   */
  protected function parseHeader($header){
    
    //Call the parent method
    parent::parseHeader($header);
    
    //Parse the list of parameters
    $current_param = "";
    $tab = preg_split("#[\r\n]+#", strstr($header, "Parameters"));
    foreach($tab as $line){
    
      //Parameter name and description
      if(preg_match("#\s*\*\s*-\s*([a-z]+)\s*:\s*(.+)#i", $line, $res)){
        $current_param = $res[1];
        $this->addParam($current_param, $res[2], null, false);
      } elseif(preg_match("#Default value\s*:\s*(.+)#", $line, $res)) {
        $this->param_desc[$current_param]["def"] = $res[1];
      } elseif(preg_match("#Optional\s*:\s*(YES|NO)#", $line, $res)) {
        $this->param_desc[$current_param]["opt"] = ($res[1] == "YES");
      }    
    }  
  }  
  
  /** 
   * @brief Set the parameter description array
   * @param $name param name
   * @param $desc descriton
   * @param $default_value default value of the parameter, null if it does not have one
   * @param $is_optional true if it is optional, false if it is mandatory
   */
  protected function addParam($name, $desc, $default_value = null, $is_optional = false){
    $this->param_desc[$name] = array("desc" => $desc, "opt" => $is_optional, "def" => $default_value);
    $this->param_order[$this->param_current++] = $name;
  }
  
  /**
   * @brief Serialize the exploit
   * @return external representation of the exploit
   */
  public function serialize(){
  
    //Get module properties
    $ser = parent::serialize();
    
    //Add id
    $ser["id"] = $this->id;
    
    //Add params
    $ser["params"] = array();
    foreach($this->param_order as $name){
      $ser["params"][] = array($name, $this->param_desc[$name]);
    }
    
    return $ser;
  }
  

  /**
   * @brief Get a registered exploit.
   * @return Exploit object corresponding to the given id, of null if the id is incorrect.
   */
  public static function getExploit($id){
    $res = null;
    if(array_key_exists($id, static::$exploit_list)){
      $res = static::$exploit_list[$id];
    }
    return $res;
  }
  
  /**
   * @brief Check the parameters
   * @param $params associative array (param name => value)
   * throw an exception if there is a problem
   */
  public function checkParams($params){
    if(!is_object($params)) $params = new stdClass();
    
    //If there is one missing parameter that is not optional and which doesn't have a default value, throw an exception
    foreach($this->param_desc as $param => $props){
      if(!property_exists($params, $param) && !$props["opt"] && $props["def"] == null){
        throw new MissingExploitParameter($param);
      }
    }
  }
  
  
  /**
   * @brief Run the exploit by generating its JS code
   * @param $token instruction token
   * @param $params exploit parameters recorded in the instruction of throw an exception if there is a missing parameter
   * @return code
   */
  public function getJSCode($token, $params){
    
    //Handle missing params
    foreach($this->param_desc as $param => $props){
      if(!property_exists($params, $param) && !$props["opt"]){
        if($props["def"] != null){
          $params->$param = $props["def"];
        } else {
          throw new MissingExploitParameter($param);
        }
      }
    }
    
    $paramsJSON = Core::getInstance()->json->encode($params);
    return "XeekAgent.Modules.Exploits[\"".$this->id."\"].run('".$token."', ".$paramsJSON.");\n";
  }
  
  /**
   * @brief Get the JS code necessary to run the exploit
   * @return JS code
   */
  public function getJSLibCode() {
    $code = file_get_contents(dirname(__FILE__)."/../modules/exploits/".$this->js_file_path);
    $code = preg_replace("#CREATE_XEEK_EXPLOIT\(#", "CREATE_XEEK_EXPLOIT(\"".$this->id."\", ", $code);
    $code = preg_replace("#THIS_XEEK_EXPLOIT\(\)#", "XEEK_EXPLOIT(\"".$this->id."\")", $code);
    return $code;
  }
  
  
  /**
   * @brief Retrieve the exploit list
   * @return exploit list
   */
  public static function getExploitList(){
    return array_values(static::$exploit_list);
  }
  
  
  /**
   * @brief Register an exploit
   * @param $filename exploit file name
   */
  public static function register($filename){
    if(!array_key_exists($filename, self::$exploit_list)){
      $e = new Exploit($filename);
      static::$exploit_list[$e->id] = $e;
    }
  }
  
}

?>