<?php

/**
 * @class Queue
 * @brief Wrapper for an instruction pile. Manages the victims' instruction flow.
 */
class Queue {

  //Reference to the audit
  protected $audit;

  /**
   * @brief Constructor
   * @param $audit Audit object associated to the queue
   */
  public function __construct(Audit $audit) {
    $this->audit = $audit;
  }

  /**
   * @brief Pop next instructions to execute
   * @param $session_id session ID
   * @param $min_id minimum instruction id to get
   * @return the list of next instructions to execute, empty array if there is not anymore
   */
  public function popNextInstructions($session_id, $min_id) {
    $this->checkAudit();
    $min_id = intval($min_id);
    $session_id = intval($session_id);
    $cond = "audit_id = '".$this->audit->id."' AND (session_id = '0' OR session_id = '".$session_id."') AND id > '".$min_id."'";
    return Instruction::fetchList($cond, "prio DESC, id ASC");
  }

  /**
   * @brief Push an instruction on top of the queue
   * @param $exploit_id name of the exploit
   * @param $params array of parameters
   * @param $session_id session ID, 0 for all sessions
   * @param $prio priority level
   * @return instruction or throw an exception if there is an error
   */
  public function pushInstr($exploit_id, $params, $session_id = 0, $prio = Instruction::PRIO_DEFAULT) {
    $this->checkAudit();
    $i = new Instruction(array( "audit_id" => $this->audit->id, 
                                "session_id" => $session_id, 
                                "exploit_id" => $exploit_id, 
                                "params" => $params, 
                                "prio" => $prio));
    $i->save();
    
    return $i;
  }
  
  /**
   * @brief Push a chain of instructions on top of the queue
   * @param $chain exploit chain
   * @param $params array of parameters for each exploit
   * @param $prio priority level for the whole chain
   * @return array containing the new instructions that have been pushed
   */
  public function pushChain(Chain $chain, $params = array(), $prio = Instruction::PRIO_DEFAULT) {
    
    //Get the exploits of the chain
    $exploit_ids = $chain->getExploits();
    
    //Check size of params array
    $c = count($params);
    if(count($params) != count($exploit_ids)) {
      throw new IncorrectNumberOfParameters();
    }
    
    $res = array();
    
    //Add each instruction to the queue
    for($i = 0; $i < $c; $i++){
      $res[] = $this->pushInstr($exploit_ids[$i], $params[$i], $prio);
    }
    
    return $res;
  }
  
  /**
   * @brief Serialize the queue
   * @return array of serialized instructions within the queue
   */
  public function serialize(){
    $this->checkAudit();
    $instructions = Instruction::fetchList("audit_id = '".$this->audit->id."'", "prio DESC, id ASC");
    $a = array();
    foreach($instructions as $i){
      $a[] = $i->serialize();
    }
    return $a;
  }
  
  
  /**
   * @brief Delete all instructions in the queue
   * @return true
   */
  public function clean(){
    Instruction::deleteWhere("audit_id = '".$this->audit->id."'");
    return true;
  }
  
  
  /**
   * @brief Check if the associated audit exists. Thow an exception if that is not the case.
   */
  protected function checkAudit(){
    if($this->audit->id == 0){
      throw new InvalidAudit();
    }
  }


}

?>